/* Copyright © 2005-2007  Roger Leigh <rleigh@debian.org>
 *
 * schroot is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * schroot is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 *********************************************************************/

#include <config.h>

#include <sbuild/sbuild-util.h>

#include "schroot-sbuild-session.h"

#include <cassert>
#include <cerrno>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <memory>

#include <unistd.h>

#include <syslog.h>

#include <boost/format.hpp>

using std::cout;
using std::endl;
using sbuild::_;
using sbuild::auth;
using boost::format;
using namespace schroot_sbuild;

session::session (std::string const&                  service,
                  operation                           operation,
                  sbuild::session::chroot_list const& chroots):
  sbuild::session(service, operation, chroots)
{
}

session::~session ()
{
}

sbuild::auth::status
session::get_chroot_auth_status (sbuild::auth::status status,
                                 sbuild::chroot::ptr const& chroot) const
{
  sbuild::passwd pwent("sbuild");


  status = auth::change_auth(status,
                             sbuild::session::get_chroot_auth_status(status,
                                                                     chroot));

  return status;
}
